@@ -52,12 +52,18 @@ all.equal(nested_contributions(1, c(0, -1), c(1, 2))(xna, x, w),
 all.equal(sum(nested_contributions(1, c(0, -1), c(1, 2))(xna, x, w), na.rm = TRUE),
           nested_mean(1, c(0, -1), c(1, 2))(xna, x, w, na.rm = TRUE) - 1)
 
-all.equal(sum(nested_contributions(0, c(3, -2))(xna, w, xna), na.rm = TRUE),
-          nested_mean(0, c(3, -2))(xna, w, xna, na.rm = TRUE) - 1)
+all.equal(sum(nested_contributions(0, c(3, -2))(x, w, xna), na.rm = TRUE),
+          nested_mean(0, c(3, -2))(x, w, xna, na.rm = TRUE) - 1)
 
-all.equal(sum(nested_contributions2(0, c(3, -2))(xna, w, xna), na.rm = TRUE),
-          nested_mean(0, c(3, -2))(xna, w, xna, na.rm = TRUE) - 1)
+all.equal(sum(nested_contributions2(0, c(3, -2))(x, w, xna), na.rm = TRUE),
+          nested_mean(0, c(3, -2))(x, w, xna, na.rm = TRUE) - 1)
 
-# TODO: Is this correct? I think it only happens when one set of weights is all NA
-all.equal(fisher_contributions(1:2, c(NA, NA)), c(0, 1 / 3))
-all.equal(fisher_contributions2(1:2, c(NA, NA)), c(NaN, NaN))
+all.equal(fisher_contributions(1:2, c(NA, NA)), fisher_contributions2(1:2, c(NA, NA)))
+
+all.equal(fisher_contributions(1:2, c(NA, NA)), harmonic_contributions(1:2))
+
+all.equal(nested_contributions(3, c(-1, 2), c(0.75, NA))(x),
+          nested_contributions2(3, c(-1, 2), c(0.75, NA))(x))
+
+all.equal(nested_contributions(3, c(-1, 2), c(0.75, NA))(x),
+          harmonic_contributions(x))
